// =========================================================================
//
// DOM.js - a W3C compliant DOM parser for XML for <SCRIPT>
//
// version 2.1
//
// =========================================================================
//
// Copyright (C) 2002, 2003 David Joham (djoham@yahoo.com) and Scott Severtson
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// visit the XML for <SCRIPT> home page at xmljs.sourceforge.net
//





DOMDriver = function() {
	this._p = null;
};
DOMDriver.prototype.parse = function(strD) {
	this._p = new XMLP(strD);
	this._doc = new DOMDocument();
	this._iNodeTop = this._doc._id;


	this._parseLoop();

	return this._doc;
};
DOMDriver.prototype._parseLoop = function() {
	var iEvt, doc = this._doc, p = this._p, iNode, iAttr, strName;

	p = this._p;
	var doodoo = true;
	while(true) {
		iEvt = p.next();

		if (XMLP._ELM_B   == iEvt) {
			iNode = doc._createElement(p.getName());
			for(var i = 0; i < p.getAttributeCount(); i++) {
				strName = p.getAttributeName(i);
				iAttr = doc._getNamedItem(iNode, strName);
				if(iAttr == -1)
					iAttr = doc._createAttribute(strName);
				doc._setNodeValue(iAttr, p.getAttributeValue(i));
				doc._setNamedItem(iNode, iAttr);
			}
			doc._appendChild(this._iNodeTop, iNode);
			this._iNodeTop = iNode;
		} 
		else if(XMLP._ELM_E   == iEvt) {
			this._iNodeTop = doc._getNodeParent(this._iNodeTop);
			break;
		} 
		
		else if(XMLP._ELM_EMP == iEvt) {
			iNode = doc._createElement(p.getName());
			for(var i = 0; i < p.getAttributeCount(); i++) {
				strName = p.getAttributeName(i);
				iAttr = doc._getNamedItem(iNode, strName);
				if(iAttr == -1)
					iAttr = doc._createAttribute(strName);
				doc._setNodeValue(iAttr, p.getAttributeValue(i));
				doc._setNamedItem(iNode, iAttr);
			}
			doc._appendChild(this._iNodeTop, iNode);
		} 
		else if(XMLP._TEXT    == iEvt) {
			doc._appendChild(this._iNodeTop, iNode = doc._createTextNode(p.getContent().substring(p.getContentBegin(), p.getContentEnd())));
		} 
		else if(XMLP._ENTITY  == iEvt) {
			doc._appendChild(this._iNodeTop, iNode = doc._createTextNode(p.getContent().substring(p.getContentBegin(), p.getContentEnd())));
		} 
		else if(XMLP._PI      == iEvt) {
			doc._appendChild(this._iNodeTop, doc._createProcessingInstruction(p.getName(), p.getContent().substring(p.getContentBegin(), p.getContentEnd())));
		} 
		else if(XMLP._CDATA   == iEvt) {
			doc._appendChild(this._iNodeTop, doc._createCDATASection(p.getContent().substring(p.getContentBegin(), p.getContentEnd())));
		} 
		else if(XMLP._COMMENT == iEvt) {
			doc._appendChild(this._iNodeTop, doc._createComment(p.getContent().substring(p.getContentBegin(), p.getContentEnd())));
		} 
		else if(XMLP._DTD     == iEvt) {
		} 
		else if(XMLP._ERROR   == iEvt) {
			alert("Fatal Error: " + p.getContent() + "\nLine: " + p.getLineNumber() + "\nColumn: " + p.getColumnNumber() + "\n");
		} else if(XMLP._NONE    == iEvt) {
		}
	}
};

///////////////
//  DOMNode  //
///////////////
DOMNode = function(doc, id) {
	this._doc = doc;
	this._id = id;
};
DN_p = DOMNode.prototype;
DOMNode.ELEMENT_NODE                = 1;
DOMNode.ATTRIBUTE_NODE              = 2;
DOMNode.TEXT_NODE                   = 3;
DOMNode.CDATA_SECTION_NODE          = 4;
DOMNode.ENTITY_REFERENCE_NODE       = 5;
DOMNode.ENTITY_NODE                 = 6;
DOMNode.PROCESSING_INSTRUCTION_NODE = 7;
DOMNode.COMMENT_NODE                = 8;
DOMNode.DOCUMENT_NODE               = 9;
DOMNode.DOCUMENT_TYPE_NODE          = 10;
DOMNode.DOCUMENT_FRAGMENT_NODE      = 11;
DOMNode.NOTATION_NODE               = 12;

DN_p.setNodeValue = function(value) { this._doc.setNodeValue(this._id, value) };

DN_p.getNodeValue     = function() { return this._doc._getNodeValue(this._id);                      };
DN_p.getNodeName      = function() { return this._doc._getNodeName (this._id);                      };
DN_p.getNodeType      = function() { return this._doc._getNodeType (this._id);                      };
DN_p.getParentNode    = function() { return this._doc._getNode(this._doc._getNodeParent(this._id)); };
DN_p.getOwnerDocument = function() { return this._doc;                                              };

DN_p.getAttributes    = function() { return new DOMNamedNodeMap(this._doc, this._id);        };
DN_p.getChildNodes    = function() { return new DOMNodeList(this._doc._getNodeChildNodes()); };
DN_p.hasChildNodes    = function() { return this._doc._hasChildNodes(this._id);              };

DN_p.getFirstChild      = function() { return this._doc._getNode(this._doc._getFirstChild     (this._id)); };
DN_p.getLastChild       = function() { return this._doc._getNode(this._doc._getLastChild      (this._id)); };
DN_p.getPreviousSibling = function() { return this._doc._getNode(this._doc._getPreviousSibling(this._id)); };
DN_p.getNextSibling     = function() { return this._doc._getNode(this._doc._getNextSibling    (this._id)); };

DN_p.insertBefore  = function(newChild, refChild) { this._doc._insertBefore(this._id, newChild._id, refChild._id); };
DN_p.replaceChild  = function(newChild, oldChild) { this._doc._replaceChild(this._id, newChild._id, oldChild._id); };
DN_p.removeChild   = function(oldChild)           { this._doc._removeChild(this._id, oldChild._id);                 };
DN_p.appendChild   = function(newChild)           { this._doc._appendChild(this._id, newChild._id);                 };

DN_p.cloneNode = function(deep) { return this._doc.getNode(this._doc._cloneNode(this._id, deep)); };
DN_p.toString = function() { return this._doc._toString(this._id); };
delete DN_p;

///////////////////
//  DOMDocument  //
///////////////////
DOMDocument = function() {
	var id;
	this._nodes = new Array();

	id = this._createDocument();

	this.DOMDocument = DOMNode;
	this.DOMDocument(this, id);

	this._docelement = null;
};
DOMDocument.escapeString = function(v) {
	v = Strings.replace(v, null, null, "&",  "&amp;" );
	v = Strings.replace(v, null, null, "<",  "&lt;"  );
	v = Strings.replace(v, null, null, ">",  "&gt;"  );
	v = Strings.replace(v, null, null, "\"", "&quot;");
	v = Strings.replace(v, null, null, "'",  "&apos;");
	return v;
};

DD_p = DOMDocument.prototype = new DOMNode();

DD_p.createDocumentFragment      = function()           { return this._getNode(this._createDocumentFragment());                };
DD_p.createElement               = function(name)       { return this._getNode(this._createElement(name));                     };
DD_p.createAttribute             = function(name)       { return this._getNode(this._createAttribute(name));                   };
DD_p.createEntityReference       = function(name)       { return this._getNode(this._createEntityReference(name));             };
DD_p.createTextNode              = function(data)       { return this._getNode(this._createTextNode(data));                    };
DD_p.createComment               = function(data)       { return this._getNode(this._createComment(data));                     };
DD_p.createCDATASection          = function(data)       { return this._getNode(this._createCDATASection(data));                };
DD_p.createProcessingInstruction = function(name, data) { return this._getNode(this._createProcessingInstruction(name, data)); };

DD_p.getElementsByTagName = function(name) { return new DOMNodeList(this._getElementsByTagName(this._id, name)); };

DD_p.getImplementation  = function() { return this;                            };
DD_p.getDocumentElement = function() { return this._getNode(this._docelement); };

DD_p.hasFeature = function(feature, version) { return ((feature.toLowerCase() == "xml") && (!version || (version == "1.0"))); };

DD_p.getDoctype = function() { alert("DOMDocument.getDoctype(): Not Implemented"); return null; };

///////////////////////////
//  DOMDocument Helpers  //
///////////////////////////
DOMDocument._NODE_TYPE       = 0;
DOMDocument._NODE_NAME       = 1;
DOMDocument._NODE_VALUE      = 2;
DOMDocument._NODE_PARENT     = 3;
DOMDocument._NODE_CHILDREN   = 4;
DOMDocument._NODE_ATTRIBUTES = 5;

DD_p._create = function(type, name, value) {
	// TODO: Look for holes left by _delete
	var array = new Array(2), id, node;
	array[DOMDocument._NODE_TYPE] = type;
	array[DOMDocument._NODE_NAME] = name;

	if(arguments.length >= 3)
		array[DOMDocument._NODE_VALUE] = value;

	id = this._nodes.length;
	this._nodes[id] = array;
	return id;
};

DD_p._delete = function(id) {
	// Kill off node & children recursively (depth first), but don't shrink array.
	var array = this._getNodeChildNodes(id), i;
	for(i = 0; i < array.length; i++)
		this._delete(array[i]);

	this._nodes[id] = null;
};

DD_p._setNodeData = function(id, field, value) {
	if((id < 0) || (id >= this._nodes.length)) return;

	var array = this._nodes[id];
	if(array)
		array[field] = value;
};
DD_p._getNodeData = function(id, field) {
	if((id < 0) || (id >= this._nodes.length)) return null;

	var array = this._nodes[id];
	if(array)
		if(field < array.length)
			return array[field];
	return null;
};

DD_p._getNode = function(id) {
	if(!this._nodes[id]) return null;

	var iType = this._getNodeType(id);
	var node = null;

	     if(DOMNode.ELEMENT_NODE                == iType) node = new DOMElement              (this, id);
	else if(DOMNode.ATTRIBUTE_NODE              == iType) node = new DOMAttr                 (this, id);
	else if(DOMNode.TEXT_NODE                   == iType) node = new DOMText                 (this, id);
	else if(DOMNode.CDATA_SECTION_NODE          == iType) node = new DOMCDATASection         (this, id);
	else if(DOMNode.ENTITY_REFERENCE_NODE       == iType) node = new DOMEntityReference      (this, id);
	else if(DOMNode.ENTITY_NODE                 == iType) node = new DOMEntity               (this, id);
	else if(DOMNode.PROCESSING_INSTRUCTION_NODE == iType) node = new DOMProcessingInstruction(this, id);
	else if(DOMNode.COMMENT_NODE                == iType) node = new DOMComment              (this, id);
	else if(DOMNode.DOCUMENT_NODE               == iType) node = new DOMDocument             (this, id);
	else if(DOMNode.DOCUMENT_TYPE_NODE          == iType) node = new DOMDocumentType         (this, id);
	else if(DOMNode.DOCUMENT_FRAGMENT_NODE      == iType) node = new DOMDocumentFragment     (this, id);
	else if(DOMNode.NOTATION_NODE               == iType) node = new DOMNotation             (this, id);

	return node;
};
DD_p._toString = function(id, buffer) {
	if(!this._nodes[id]) return null;
	var b = buffer;
	if(!b) b = new StringBuffer();

	var iType = this._getNodeType(id);
	var array, v;

	if(DOMNode.ELEMENT_NODE == iType) {
		v = this._getNodeName(id);
		b.append("<");
		b.append(v);

		array = this._getNodeAttributes(id);
		for(var i = 0; i < array.length; i++) {
			b.append(" ");
			this._toString(array[i], b);
		}
		b.append(">");

		array = this._getNodeChildNodes(id);
		for(var i = 0; i < array.length; i++) {
			this._toString(array[i], b);
		}

		b.append("</");
		b.append(v);
		b.append(">");
	} else if(DOMNode.ATTRIBUTE_NODE == iType) {
		v = this._getNodeValue(id);
		b.append(this._getNodeName(id));
		if(v != null) {
			b.append("=\"");
			b.append(DOMDocument.escapeString(v));
			b.append("\"");
		}
	} else if(DOMNode.TEXT_NODE == iType) {
		b.append(DOMDocument.escapeString(this._getNodeValue(id)));
	} else if(DOMNode.CDATA_SECTION_NODE == iType) {
		b.append("<[CDATA[");
		b.append(this._getNodeValue(id));
		b.append("]]>");
	} else if(DOMNode.PROCESSING_INSTRUCTION_NODE == iType) {
		b.append("<?");
		b.append(this._getNodeName(id));
		b.append(" ");
		b.append(this._getNodeValue(id));
		b.append("?>");
	} else if(DOMNode.COMMENT_NODE == iType) {
		b.append("<!--");
		b.append(this._getNodeValue(id));
		b.append("-->");
	} else if((DOMNode.DOCUMENT_NODE == iType)
	       || (DOMNode.DOCUMENT_FRAGMENT_NODE == iType)) {
		array = this._getNodeChildNodes(id);
		for(var i = 0; i < array.length; i++) {
			this._toString(array[i], b);
		}
	}
	
	return buffer ? null : b.toString();
}
DD_p._createDocument              = function()           { return this._create(DOMNode.DOCUMENT_NODE              , "#document"               ); };
DD_p._createDocumentFragment      = function()           { return this._create(DOMNode.DOCUMENT_FRAGMENT_NODE     , "#document-fragment"      ); };
DD_p._createElement               = function(name)       { return this._create(DOMNode.ELEMENT_NODE               , name                      ); };
DD_p._createAttribute             = function(name)       { return this._create(DOMNode.ATTRIBUTE_NODE             , name                      ); };
DD_p._createEntityReference       = function(name)       { return this._create(DOMNode.ENTITY_REFERENCE_NODE      , name                      ); };
DD_p._createTextNode              = function(data)       { return this._create(DOMNode.TEXT_NODE                  , "#text"             , data); };
DD_p._createComment               = function(data)       { return this._create(DOMNode.COMMENT_NODE               , "#comment"          , data); };
DD_p._createCDATASection          = function(data)       { return this._create(DOMNode.CDATA_SECTION_NODE         , "#cdata-section"    , data); };
DD_p._createProcessingInstruction = function(name, data) { return this._create(DOMNode.PROCESSING_INSTRUCTION_NODE, name                , data); };


///////////////////////
//  DOMNode Helpers  //
///////////////////////
DD_p._setNodeValue      = function(id, value) { return this._setNodeData(id, DOMDocument._NODE_VALUE, value);              };
DD_p._getNodeType       = function(id)        { return this._getNodeData(id, DOMDocument._NODE_TYPE);                      };
DD_p._getNodeName       = function(id)        { return this._getNodeData(id, DOMDocument._NODE_NAME)       || null;        };
DD_p._getNodeValue      = function(id)        { return this._getNodeData(id, DOMDocument._NODE_VALUE)      || null;        };
DD_p._getNodeParent     = function(id)        { return this._getNodeData(id, DOMDocument._NODE_PARENT)     || -1;          };
DD_p._getNodeChildNodes = function(id)        { return this._getNodeData(id, DOMDocument._NODE_CHILDREN)   || new Array(); };
DD_p._getNodeAttributes = function(id)        { return this._getNodeData(id, DOMDocument._NODE_ATTRIBUTES) || new Array(); };


DD_p._getFirstChild = function(id) {
	var array = this._getNodeChildNodes(id);
	return (array && array.length > 0) ? array[0] : -1;
};
DD_p._getLastChild = function(id) {
	var array = this._getNodeChildNodes(id);
	return (array && array.length > 0) ? array[array.length - 1] : -1;
};
DD_p._getPreviousSibling = function(id) {
	var array = this._getNodeChildNodes(this._getNodeParent(id));
	var index = this._findChildIndex(this._getNodeParent(id), id);

	index--;

	return ((index >= 0) || (index < array.length)) ? array[index] : -1;
};
DD_p._getNextSibling = function(id) {
	var array = this._getNodeChildNodes(this._getNodeParent(id));
	var index = this._findChildIndex(this._getNodeParent(id), id);

	index++;

	return ((index >= 0) || (index < array.length)) ? array[index] : -1;
};

DD_p._insertBefore = function(id, newChildId, refChildId) {
	if((id == this._id) && (this._getNodeType(newChildId) == DOMNode.ELEMENT_NODE)) {
		if (this._docelement == null) {
			this._docelement = newChildId;
		} else {
			alert("DOMDocument.appendChild(): Document must only contain one child element")
			return;
		}
	}

	var array = this._getNodeChildNodes(this._getNodeParent(id));
	var index = this._findChildIndex(id, refChildId);
	if(index >= 0) {
		this._setNodeData(id, DOMDocument._NODE_CHILDREN, array.concat(array.slice(0, index), newChildId, array.slice(index)));
		this._setNodeData(newChildID, DOMDocument._NODE_PARENT, id);
	}
};
DD_p._replaceChild = function(id, newChildId, oldChildId) {
	var array = this._getNodeChildNodes(this._getNodeParent(id));
	var index = this._findChildIndex(id, oldChildId);
	if(index >= 0) {
		this._setNodeData(oldChildId, DOMDocument._NODE_PARENT, null);
		array[index] = newChildId;
		this._setNodeData(id, DOMDocument._NODE_CHILDREN, array);
		this._setNodeData(newChildId, DOMDocument._NODE_PARENT, id);
	}
};
DD_p._removeChild = function(id, oldChildId) {
	var array = this._getNodeChildNodes(this._getNodeParent(id));
	var index = this._findChildIndex(id, oldChildId);
	if(index >= 0) {
		this._setNodeData(id, DOMDocument._NODE_CHILDREN, array.slice(0, index).concat(array.slice(index + 1)));
		this._setNodeData(oldChildId, DOMDocument._NODE_PARENT, null);
	}
};
DD_p._appendChild = function(id, newChildId) {
	if((id == this._id) && (this._getNodeType(newChildId) == DOMNode.ELEMENT_NODE)) {
		if (this._docelement == null) {
			this._docelement = newChildId;
		} else {
			alert("DOMDocument.appendChild(): Document must only contain one child element")
			return;
		}
	}

	var array = this._getNodeChildNodes(id);
	array[array.length] = newChildId;
	this._setNodeData(newChildId, DOMDocument._NODE_PARENT, id);
	this._setNodeData(id, DOMDocument._NODE_CHILDREN, array);
};
DD_p._hasChildNodes = function(id) {
	return (this._getNodeChildNodes(id).length != 0);
};
DD_p._findChildIndex = function(id, childId) {
	var array = this._getNodeChildNodes(id);
	for(var i = 0; i < array.length; i++)
		if(array[i] == id)
			return i;
	return -1;
};
DD_p._cloneNode = function(id, deep) {
	var cloneId, array;

	cloneId = this._create(this._nodes[id][DOMDocument._NODE_TYPE], this._nodes[id][DOMDocument._NODE_NAME]);
	this._setNodeValue(cloneId, this._getNodeValue(id));

	array = this._getNodeAttributes(id);
	for(i = 0; i < array.length; i++) {
		this._setAttributeNode(cloneId, this._cloneNode(array[i], false));
	}


	if(deep) {
		array = this._getNodeChildNodes(id);
		for(i = 0; i < array.length; i++) {
			this._appendChild(cloneId, this._cloneNode(array[i], true));
		}

	}

	return cloneId;

};
//////////////////////////
//  DOMElement Helpers  //
//////////////////////////
DD_p._getElementsByTagName = function(id, tagname) {
	return this._getElementsByTagNameRecursive(id, tagname, new Array());
};
DD_p._getElementsByTagNameRecursive = function(id, tagname, array) {
	var arrChildren = this._getNodeChildNodes(id);
	if(this._getNodeType(id) == DOMNode.ELEMENT_NODE) {

		if((this._getNodeName(id) == tagname) || (tagname == "*")) {
			array[array.length] = id;
		}

		for(var i = 0; i < arrChildren.length; i++) {
			array = this._getElementsByTagNameRecursive(arrChildren[i], tagname, array);
		}
	}

	return array;
};
DD_p._normalize = function(id) {
	var i, array, nodeId, nodeIdLastText, arrToDelete = new Array();;

	array = this._getNodeChildNodes(id);
	for(i = 0; i < array.length; i++) {
		nodeId = array[i];
		if(this._getNodeType(nodeId) == DOMNode.TEXT_NODE) {
			if(nodeIdLastText) {
				this._appendNodeValue(nodeIdLastText, this._getNodeValue(nodeId));
				arrToDelete[arrToDelete.length] = nodeId;
			} else {
				nodeIdLastText = node;
			}
		} else {
			nodeIdLastText = null;
		}
	}

	for(i = 0; i < arrToDelete.length; i++)
		this._delete(arrToDelete[i]);
};
///////////////////////////////
//  DOMNamedNodeMap Helpers  //
///////////////////////////////
DD_p._getNamedItem = function(id, name) {
	return this._findNamedItemId(id, name);
};
DD_p._setNamedItem = function(id, itemId) {
	var array = this._getNodeAttributes(id);
	var index = this._findNamedItemIndex(id, this._getNodeName(itemId));
	if(index >= 0) {
		array[index] = itemId;
	} else {
		array[array.length] = itemId;
	}
	this._setNodeData(id, DOMDocument._NODE_ATTRIBUTES, array);
	this._setNodeData(itemId, DOMDocument._NODE_PARENT, id);
};
DD_p._removeNamedItem = function(id, name) {
	var array = this._getNodeAttributes(id);
	var index = this._findNamedItemIndex(id, name);

	if(index >= 0) {
		this._setNodeData(array[index], DOMDocument._NODE_PARENT, null);
		this._setNodeData(id, DOMDocument._NODE_CHILDREN, array.slice(0, index).concat(array.slice(index + 1)));
	}
};
DD_p._findNamedItemIndex = function(id, name) {
	var array = this._getNodeAttributes(id);
	for(var i = 0; i < array.length; i++)
		if(this._getNodeName(array[i]) == name)
			return i;
	return -1;
};
DD_p._findNamedItemId = function(id, name) {
	var array = this._getNodeAttributes(id);
	for(var i = 0; i < array.length; i++)
		if(this._getNodeName(array[i]) == name)
			return array[i];
	return -1;
};

/////////////////////////////////
//  DOMCCharacterData Helpers  //
/////////////////////////////////
DD_p._substringNodeValue = function(id, offset, count) {
	var value = this._getNodeValue(id);
	return value ? value.substring(offset, count) : null;
};
DD_p._appendNodeValue = function(id, arg) {
	var value = this._getNodeValue(id);
	this._setNodeValue(id, (value || "").concat(arg));
};
DD_p._inserNodeValue = function(id, offset, arg) {
	var value = this._getNodeValue(id);
	if(value)
		value = value.substring(0, offset).concat(arg, value.substring(offset));
	else
		value = arg;

	this._setNodeValue(id, value);
};
DD_p._deleteNodeValue = function(id, offset, count) {
	var value = this._getNodeValue(id);
	if(value) {
		if(!count || (offset + count) > this._nodeValue.length) {
			value = value.substring(0, offset);
		} else {
			value = value.substring(0, offset).concat(value.substring(offset + count));
		}
		this._setNodeValue(id, value);
	}
};
DD_p._replaceNodeValue = function(id, offset, count, arg) {
	var value = this._getNodeValue(id);
	if(value)
		value = value.substring(0, offset).concat(arg, value.substring(offset + count));
	else
		value = arg;
	this._setNodeValue(id, value);
};
DD_p._getNodeValueLength = function(id) {
	var value = this.getNodeValue(id);
	return value ? value.length : 0;
};

///////////////////////
//  DOMText Helpers  //
///////////////////////
DD_p._splitNodeValue = function(id, offset) {
	var data, nodeID, parentID;
	parentID = this._getParent(id);
	if(parentID >= 0) {
		data = this._substringNodeValue(id, offset);
		nodeID = this._createTextNode(data);

		this._insertBefore(parentID, nodeID, this._nextSibling(id));

		this._deleteNodeValue(id, offset);
	}
};
delete DD_p;

///////////////////
//  DOMNodeList  //
///////////////////
DOMNodeList = function(array) {
	this._array = array;
};
DOMNodeList.prototype.item = function(index) {
	return ((index >= this._array.length) || (index < 0)) ? null : this._array[index];
};
DOMNodeList.prototype.getLength = function() {
	return this._array.length;
};

///////////////////////
//  DOMNamedNodeMap  //
///////////////////////
DOMNamedNodeMap = function(doc, id) {
	this._doc = doc;
	this._id = id;
};
DOMNamedNodeMap.prototype.getNamedItem = function(name) {
	return this._doc._getNamedItem(this._id, name);
};
DOMNamedNodeMap.prototype.setNamedItem = function(arg) {
	this._doc._setNamedItem(this._id, arg._id);
};
DOMNamedNodeMap.prototype.removeNamedItem = function(name) {
	this._doc._removeNamedItem(this._id, name);
};
DOMNamedNodeMap.prototype.getLength = function() {
	var array = this._doc._getNodeAttributes(this._id);
	return array.length;
};
DOMNamedNodeMap.prototype.item = function(index) {
	var array = this._doc._getNodeAttributes(this._id);
	return ((index >= 0) || (index < array.length)) ? array[index] : null;
};



//////////////////
//  DOMElement  //
//////////////////
DOMElement = function(doc, id) {
	this.DOMElement = DOMNode;
	this.DOMElement(doc, id);
};
DE_p = DOMElement.prototype = new DOMNode();
DE_p.getTagName      = function()            { return                            this._doc._getNodeValue   (this._id                );  };
DE_p.getAttribute    = function(name)        { return this._doc._getNodeValue   (this._doc._getNamedItem   (this._id, name          )); };
DE_p.setAttribute    = function(name, value) {
	var iAttr = this._doc._getNamedItem(this._id, name);
	if(iAttr == -1)
		iAttr = this._doc._createAttribute(name);
	this._doc._setNodeValue(iAttr, value);
	this._doc._setNamedItem(this._id, iAttr);
};
DE_p.removeAttribute = function(name)        {                                   this._doc._removeNamedItem(this._id, name          );  };

DE_p.getAttributeNode    = function(name)    { return this._doc._getNode(this._doc._getNamedItem   (this._id, name             )); };
DE_p.setAttributeNode    = function(newAttr) {                           this._doc._setNamedItem   (this._id, newAttr._id     );  };
DE_p.removeAttributeNode = function(oldAttr) {                           this._doc._removeNamedItem(this._id, oldAttr.getName());  };

DE_p.getElementsByTagName = function(tagname) { return new DOMNodeList(this._doc._getElementsByTagName(this._id, tagname)); };
DE_p.normalize            = function()        { this._doc._normalize(this._id);                                             };


////////////////////////
//  DOMCharacterData  //
////////////////////////
DOMCharacterData = function(doc, id) {
	this.DOMText = DOMNode;
	this.DOMText(doc, id);
};
DCD_p = DOMCharacterData.prototype = new DOMNode();
DCD_p.getData   = function()     { return this._doc._getNodeValue(this._id);       };
DCD_p.setData   = function(data) { this._doc._setNodeValue(this._id, data);        };
DCD_p.getLength = function()     { return this._doc._getNodeValueLength(this._id); };

DCD_p.substringData = function(offset, count)      { return this._doc._substringNodeValue(this._id, offset, count     ); };
DCD_p.appendData    = function(arg)                {        this._doc._appendNodeValue   (this._id, arg               ); };
DCD_p.insertData    = function(offset, arg)        {        this._doc._insertNodeValue   (this._id, offset, arg       ); };
DCD_p.deleteData    = function(offset, count)      {        this._doc._deleteNodeValue   (this._id, offset, count     ); };
DCD_p.replaceData   = function(offset, count, arg) {        this._doc._replaceNodeValue  (this._id, offset, count, arg); };
delete DCD_p;

///////////////
//  DOMText  //
///////////////
DOMText = function(doc, id) {
	this.DOMText = DOMCharacterData;
	this.DOMText(doc, id);
};
DT_p = DOMText.prototype = new DOMCharacterData();
DT_p.splitText = function(offset) { this._doc._splitNodeValue(this._id, offset); };
delete DT_p;

///////////////////////
//  DOMCDATASection  //
///////////////////////
DOMCDATASection = function(doc, id) {
	this.DOMCDATASection = DOMCharacterData;
	this.DOMCDATASection(doc, id);
};
DCDS_p = DOMCDATASection.prototype = new DOMCharacterData();
DCDS_p.splitText = function(offset) { this._doc._splitNodeValue(this._id, offset); };
delete DCDS_p;

//////////////////
//  DOMComment  //
//////////////////
DOMComment = function(doc, id) {
	this.DOMComment = DOMCharacterData;
	this.DOMComment(doc, id);
};
DC_p = DOMComment.prototype = new DOMCharacterData();
delete DC_p;

///////////////
//  DOMAttr  //
///////////////
DOMAttr = function(doc, id) {
	this.DOMAttr = DOMNode;
	this.DOMAttr(doc, id);
};
DA_p = DOMAttr.prototype = new DOMNode();
DA_p.getName      = function()      { return this._doc._getNodeName(this._id);          };
DA_p.getSpecified = function()      { return this._doc._getNodeValue(this._id) != null; };
DA_p.getValue     = function()      { return this._doc._getNodeValue(this._id);         };
DA_p.setValue     = function(value) {        this._doc._setNodeValue(this._id, value);  };
delete DA_p;

///////////////////////////
//  DOMDocumentFragment  //
///////////////////////////

DOMDocumentFragment = function(doc, id) {
	this.DOMDocumentFragment = DOMNode;
	this.DOMDocumentFragment(doc, id);
};
DDF_p = DOMDocumentFragment.prototype = new DOMNode();
delete DDF_p;

////////////////////////////////
//  DOMProcessingInstruction  //
////////////////////////////////
DOMProcessingInstruction = function(doc, id) {
	this.DOMProcessingInstruction = DOMNode;
	this.DOMProcessingInstruction(doc, id);
};
DPI_p = DOMProcessingInstruction.prototype = new DOMNode();
DPI_p.getTarget = function()     { return this._doc._getNodeName(this._id);        };
DPI_p.getData   = function()     { return this._doc._getNodeValue(this._id);       };
DPI_p.setData   = function(data) { return this._doc._setNodeValue(this._id, data); };
delete DPI_p;

///////////////////////
//  NOT IMPLEMENTED  //
///////////////////////
DOMDocumentType    = function() { alert("DOMDocumentType.constructor(): Not Implemented"   ); };
DOMEntity          = function() { alert("DOMEntity.constructor(): Not Implemented"         ); };
DOMEntityReference = function() { alert("DOMEntityReference.constructor(): Not Implemented"); };
DOMNotation        = function() { alert("DOMNotation.constructor(): Not Implemented"       ); };

